@@ -3,7 +3,7 @@ import numpy as np
 from gensim import corpora
 from gensim.models import LdaModel, LsiModel
 from sklearn.decomposition import NMF
-from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
+from sklearn.feature_extraction.text import TfidfVectorizer
 from typing import Dict, List
 
 
@@ -31,50 +31,21 @@ def _rank_clusters(model, text_data: List[str], vocab: corpora.Dictionary) -> tu
     return _top_topic, _distribution
 
 
-def _term_frequency_inverse_document_frequency(text_content: np.ndarray, lang: str) -> np.ndarray:
-    """
-    Apply term-frequency inverse document frequency
-
-    :param text_content: np.ndarray
-        Text data
-
-    :param lang: str
-        Abbreviated name for the stopwords language
-
-    :return: np.ndarray
-        Transformed text content using term-frequency inverse document frequency vectorizer
-    """
-    _tfidf = TfidfVectorizer(input='content',
-                             encoding='uft-8',
-                             decode_error='strict',
-                             strip_accents=None,
-                             lowercase=True,
-                             preprocessor=None,
-                             tokenizer=None,
-                             analyzer='word',
-                             stop_words=lang,
-                             ngram_range=(1, 1),
-                             vocabulary=None,
-                             binary=False,
-                             norm='l2',
-                             use_idf=True,
-                             smooth_idf=True,
-                             sublinear_tf=False
-                             )
-    return _tfidf.fit_transform(raw_documents=text_content)
-
-
 class GibbsSamplingDirichletMultinomialModeling:
     """
     Class for building Gibbs Sampling Dirichlet Multinomial Modeling model
     """
     def __init__(self,
+                 vocab_size: int,
                  n_clusters: int = 8,
                  n_iterations: int = 30,
                  alpha: float = 0.1,
                  beta: float = 0.1
                  ):
         """
+        :param vocab_size: int
+            Size of the vocabulary
+
         :param n_clusters: int
             Number of pre-defined clusters
 
@@ -92,7 +63,7 @@ class GibbsSamplingDirichletMultinomialModeling:
         self.n_clusters: int = n_clusters
         self.n_iterations: int = n_iterations if n_iterations > 5 else 30
         self.n_documents: int = 0
-        self.vocab_size: int = 0
+        self.vocab_size: int = vocab_size
         self.probability_vector: List[float] = []
         self.cluster_word_count: List[int] = [0 for _ in range(0, self.n_clusters, 1)]
         self.cluster_document_count: List[int] = [0 for _ in range(0, self.n_clusters, 1)]
@@ -140,33 +111,16 @@ class GibbsSamplingDirichletMultinomialModeling:
         """
         return [i for i, entry in enumerate(np.random.multinomial(1, self.probability_vector)) if entry != 0][0]
 
-    def doc_to_label(self, documents: List[str]) -> tuple:
-        """
-        Match highest probability label to document
-
-        :param documents: List[str]
-            The document token stream
-
-        :return: tuple
-            Label and the highest probability
-        """
-        _probability_vector = self._document_scoring(documents=documents)
-        return np.argmax(_probability_vector), max(_probability_vector)
-
-    def fit(self, documents: List[List[str]], vocab_size: int) -> List[int]:
+    def fit(self, documents: List[List[str]]) -> List[int]:
         """
         Fitting GSDMM clustering algorithm
 
         :param documents: List[List[str]]
             The document token stream
 
-        :param vocab_size: int
-            Size of the vocabulary
-
         :return: List[int]
             Cluster labels
         """
-        self.vocab_size = vocab_size
         self.n_documents = len(documents)
         _document_cluster: List[int] = [_ for _ in range(0, self.n_documents, 1)]
         # initialize the clusters
@@ -214,24 +168,23 @@ class GibbsSamplingDirichletMultinomialModeling:
         self.n_clusters = _n_clusters
         return _document_cluster
 
-    def generate_topic_allocation(self, documents: List[str]) -> Dict[str, List[int]]:
+    def generate_topic_allocation(self, documents: List[List[str]]) -> List[int]:
         """
-        allocates all topics to each document in original dataframe,
-        adding two columns for cluster number and cluster description
+        Allocate cluster labels (topics) to each document
+
+        :return: List[int]
+            Allocated labels
         """
-        _topic_allocations = dict(cluster=[])
+        _topic_allocations = []
         for doc in documents:
-            _topic_label, _score = self.doc_to_label(documents=[doc])
-            _topic_allocations['cluster'].append(_topic_label)
+            _topic_label = self.predict(documents=doc)
+            _topic_allocations.append(_topic_label)
         return _topic_allocations
 
-    def get_top_words_each_cluster(self, top_clusters: List[int], top_n_words: int = 5) -> dict:
+    def get_top_words_each_cluster(self, top_n_words: int = 5) -> dict:
         """
         Get top n words of each cluster
 
-        :param top_clusters: List[int]
-            Cluster indices
-
         :param top_n_words: int
             Number of top n words of each cluster
 
@@ -239,18 +192,47 @@ class GibbsSamplingDirichletMultinomialModeling:
             Top n words of each cluster
         """
         _top_n_words_each_cluster: dict = {}
-        for cluster in top_clusters:
-            _sorted_words_current_cluster = sorted(self.cluster_word_distribution[cluster].items(), key=lambda c: c[1], reverse=True)[:top_n_words]
+        for cluster in range(0, self.n_clusters, 1):
+            _sorted_words_current_cluster = sorted(self.cluster_word_distribution[cluster].items(),
+                                                   key=lambda c: c[1],
+                                                   reverse=True
+                                                   )[:top_n_words]
             _top_n_words_each_cluster.update({str(cluster): _sorted_words_current_cluster})
         return _top_n_words_each_cluster
 
-    def word_importance_each_cluster(self):
+    def predict(self, documents: List[str]) -> int:
         """
-        returns a word-topic matrix[phi] where each value represents
-        the word importance for that particular cluster;
-        phi[i][w] would be the importance of word w in topic i.
+        Predict cluster label
+
+        :param documents: List[str]
+            The document token stream
+
+        :return: int
+            Cluster label
         """
-        _phi: List[dict] = [{} for i in range(0, self.n_clusters, 1)]
+        return np.array(self._document_scoring(documents=documents)).argmax()
+
+    def predict_proba(self, documents: List[str]) -> List[float]:
+        """
+        Predict probabilities for each document
+
+        :param documents: List[str]
+            The document token stream
+
+        :return: List[np.array]
+            Probability vector for each document
+        """
+        return self._document_scoring(documents=documents)
+
+    def word_importance_each_cluster(self) -> List[dict]:
+        """
+        Generate word importance (phi) for each cluster
+
+        :return: List[dict]
+            Word importance (phi) for each cluster
+                -> phi[cluster][word]
+        """
+        _phi: List[dict] = [{} for _ in range(0, self.n_clusters, 1)]
         for c in range(0, self.n_clusters, 1):
             for w in self.cluster_word_distribution[c]:
                 _phi[c][w] = (self.cluster_word_distribution[c][w] + self.beta) / (sum(self.cluster_word_distribution[c].values()) + self.vocab_size * self.beta)
@@ -407,26 +389,56 @@ class NonNegativeMatrixFactorization:
     """
     def __init__(self,
                  lang: str,
-                 doc_term_matrix: list,
-                 vocab: corpora.Dictionary,
                  n_clusters: int = 200,
                  n_iterations: int = 10
                  ):
         """
         :param lang: str
-            Abbreviated language name
+            Full language name
 
-        :param doc_term_matrix:
-        :param vocab:
-        :param n_clusters:
-        :param n_iterations:
+        :param n_clusters: int
+            Number of clusters
+
+        :param n_iterations: int
+            Number of iterations
         """
         self.model = None
+        self.tfidf = None
         self.lang: str = lang
         self.n_clusters: int = n_clusters
         self.n_iterations: int = n_iterations
-        self.vocab: corpora.Dictionary = vocab
-        self.document_term_matrix: list = doc_term_matrix
+
+    def _term_frequency_inverse_document_frequency(self, text_content: np.ndarray, lang: str) -> np.ndarray:
+        """
+        Apply term-frequency inverse document frequency
+
+        :param text_content: np.ndarray
+            Text data
+
+        :param lang: str
+            Abbreviated name for the stopwords language
+
+        :return: np.ndarray
+            Transformed text content using term-frequency inverse document frequency vectorizer
+        """
+        self.tfidf = TfidfVectorizer(input='content',
+                                     encoding='uft-8',
+                                     decode_error='strict',
+                                     strip_accents=None,
+                                     lowercase=True,
+                                     preprocessor=None,
+                                     tokenizer=None,
+                                     analyzer='word',
+                                     stop_words=lang,
+                                     ngram_range=(1, 1),
+                                     vocabulary=None,
+                                     binary=False,
+                                     norm='l2',
+                                     use_idf=True,
+                                     smooth_idf=True,
+                                     sublinear_tf=False
+                                     )
+        return self.tfidf.fit_transform(raw_documents=text_content)
 
     def fit(self, documents: np.ndarray) -> List[int]:
         """
@@ -438,21 +450,37 @@ class NonNegativeMatrixFactorization:
         :return: List[int]
             Cluster labels
         """
-        _tfidf = _term_frequency_inverse_document_frequency(text_content=documents, lang=self.lang)
-        _nmf: NMF = NMF(n_components=self.n_clusters,
-                        init=None,
-                        solver='cd',
-                        beta_loss='frobenius',
-                        tol=0.0004,
-                        max_iter=self.n_iterations,
-                        random_state=1234,
-                        alpha=0,
-                        l1_ratio=0,
-                        verbose=0,
-                        shuffle=False
-                        )
-        _nmf.fit(X=_tfidf, y=None)
-        _cluster_labels: List[int] = []
-        for doc in documents:
-            _cluster_labels.append(_nmf.transform(_term_frequency_inverse_document_frequency(text_content=doc, lang=self.lang)).argsort(axis=1)[:, -1].tolist()[0])
+        _tfidf = self._term_frequency_inverse_document_frequency(text_content=documents, lang=self.lang)
+        self.model: NMF = NMF(n_components=self.n_clusters,
+                              init=None,
+                              solver='cd',
+                              beta_loss='frobenius',
+                              tol=0.0004,
+                              max_iter=self.n_iterations,
+                              random_state=1234,
+                              alpha=0,
+                              l1_ratio=0,
+                              verbose=0,
+                              shuffle=False
+                              )
+        self.model.fit(X=_tfidf, y=None)
+        _cluster_labels: List[int] = self.model.transform(self._term_frequency_inverse_document_frequency(text_content=documents,
+                                                                                                          lang=self.lang
+                                                                                                          )
+                                                          ).argmax(axis=1).tolist()
         return _cluster_labels
+
+    def get_top_words_each_cluster(self, top_n_words: int = 5) -> dict:
+        """
+        Get top n words of each cluster
+
+        :param top_n_words: int
+            Number of top n words of each cluster
+
+        :return: dict
+            Top n words of each cluster
+        """
+        _top_n_words_each_cluster: dict = {}
+        for cluster, topic in enumerate(self.model.components_):
+            _top_n_words_each_cluster.update({str(cluster): [self.tfidf.get_feature_names()[i] for i in topic.argsort()[-top_n_words:]]})
+        return _top_n_words_each_cluster
